@@ -76,34 +76,6 @@ static const char *type2string(enum connman_element_type type)
 	return NULL;
 }
 
-const char *__connman_ipv4_method2string(enum connman_ipv4_method method)
-{
-	switch (method) {
-	case CONNMAN_IPV4_METHOD_UNKNOWN:
-		return "unknown";
-	case CONNMAN_IPV4_METHOD_OFF:
-		return "off";
-	case CONNMAN_IPV4_METHOD_STATIC:
-		return "static";
-	case CONNMAN_IPV4_METHOD_DHCP:
-		return "dhcp";
-	}
-
-	return "unknown";
-}
-
-enum connman_ipv4_method __connman_ipv4_string2method(const char *method)
-{
-	if (strcasecmp(method, "off") == 0)
-		return CONNMAN_IPV4_METHOD_OFF;
-	else if (strcasecmp(method, "static") == 0)
-		return CONNMAN_IPV4_METHOD_STATIC;
-	else if (strcasecmp(method, "dhcp") == 0)
-		return CONNMAN_IPV4_METHOD_DHCP;
-	else
-		return CONNMAN_IPV4_METHOD_UNKNOWN;
-}
-
 static void emit_element_signal(DBusConnection *conn, const char *member,
 					struct connman_element *element)
 {
@@ -286,7 +258,7 @@ static struct connman_network *__connman_element_get_network(struct connman_elem
 
 struct connman_service *__connman_element_get_service(struct connman_element *element)
 {
-	struct connman_service *service;
+	struct connman_service *service = NULL;
 	struct connman_network *network;
 	struct connman_device *device;
 	enum connman_device_type type;
@@ -302,11 +274,12 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
 	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
 	case CONNMAN_DEVICE_TYPE_NOVATEL:
-		return NULL;
+		break;
 	case CONNMAN_DEVICE_TYPE_ETHERNET:
 		service = __connman_service_lookup_from_device(device);
 		break;
@@ -703,11 +676,11 @@ int connman_element_get_value(struct connman_element *element,
 
 	switch (id) {
 	case CONNMAN_PROPERTY_ID_IPV4_METHOD:
-		if (element->ipv4.method == CONNMAN_IPV4_METHOD_UNKNOWN)
+		if (element->ipv4.method == CONNMAN_IPCONFIG_METHOD_UNKNOWN)
 			return connman_element_get_value(element->parent,
 								id, value);
 		__connman_element_lock(element);
-		*((const char **) value) = __connman_ipv4_method2string(element->ipv4.method);
+		*((const char **) value) = __connman_ipconfig_method2string(element->ipv4.method);
 		__connman_element_unlock(element);
 		break;
 	case CONNMAN_PROPERTY_ID_IPV4_ADDRESS:
@@ -939,6 +912,7 @@ int __connman_element_append_ipv4(struct connman_element *element,
 {
 	const char *method = NULL;
 	const char *address = NULL, *netmask = NULL, *gateway = NULL;
+	const char *broadcast = NULL, *nameserver = NULL;
 
 	connman_element_get_value(element,
 				CONNMAN_PROPERTY_ID_IPV4_METHOD, &method);
@@ -949,6 +923,10 @@ int __connman_element_append_ipv4(struct connman_element *element,
 				CONNMAN_PROPERTY_ID_IPV4_NETMASK, &netmask);
 	connman_element_get_value(element,
 				CONNMAN_PROPERTY_ID_IPV4_GATEWAY, &gateway);
+	connman_element_get_value(element,
+			CONNMAN_PROPERTY_ID_IPV4_BROADCAST, &broadcast);
+	connman_element_get_value(element,
+			CONNMAN_PROPERTY_ID_IPV4_NAMESERVER, &nameserver);
 
 	if (method != NULL)
 		connman_dbus_dict_append_variant(dict, "IPv4.Method",
@@ -966,6 +944,14 @@ int __connman_element_append_ipv4(struct connman_element *element,
 		connman_dbus_dict_append_variant(dict, "IPv4.Gateway",
 						DBUS_TYPE_STRING, &gateway);
 
+	if (broadcast != NULL)
+		connman_dbus_dict_append_variant(dict, "IPv4.Broadcast",
+						DBUS_TYPE_STRING, &broadcast);
+
+	if (nameserver != NULL)
+		connman_dbus_dict_append_variant(dict, "IPv4.Nameserver",
+						DBUS_TYPE_STRING, &nameserver);
+
 	return 0;
 }
 
@@ -977,15 +963,15 @@ int __connman_element_set_ipv4(struct connman_element *element,
 	type = dbus_message_iter_get_arg_type(value);
 
 	if (g_str_equal(name, "IPv4.Method") == TRUE) {
-		enum connman_ipv4_method method;
+		enum connman_ipconfig_method method;
 		const char *str;
 
 		if (type != DBUS_TYPE_STRING)
 			return -EINVAL;
 
 		dbus_message_iter_get_basic(value, &str);
-		method = __connman_ipv4_string2method(str);
-		if (method == CONNMAN_IPV4_METHOD_UNKNOWN)
+		method = __connman_ipconfig_string2method(str);
+		if (method == CONNMAN_IPCONFIG_METHOD_UNKNOWN)
 			return -EINVAL;
 
 		if (method == element->ipv4.method)
@@ -1029,6 +1015,30 @@ int __connman_element_set_ipv4(struct connman_element *element,
 		g_free(element->ipv4.gateway);
 		element->ipv4.gateway = g_strdup(gateway);
 
+		connman_element_update(element);
+	} else if (g_str_equal(name, "IPv4.Broadcast") == TRUE) {
+		const char *broadcast;
+
+		if (type != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(value, &broadcast);
+
+		g_free(element->ipv4.broadcast);
+		element->ipv4.broadcast = g_strdup(broadcast);
+
+		connman_element_update(element);
+	} else if (g_str_equal(name, "IPv4.Nameserver") == TRUE) {
+		const char *nameserver;
+
+		if (type != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(value, &nameserver);
+
+		g_free(element->ipv4.nameserver);
+		element->ipv4.nameserver = g_strdup(nameserver);
+
 		connman_element_update(element);
 	}
 
@@ -1065,6 +1075,16 @@ static void emit_state_change(DBusConnection *conn, const char *state)
 	append_state(&entry, state);
 
 	g_dbus_send_message(conn, signal);
+
+	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "StateChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_iter_init_append(signal, &entry);
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &state);
+
+	g_dbus_send_message(conn, signal);
 }
 
 static void probe_element(struct connman_element *element)
@@ -1106,13 +1126,19 @@ static void register_element(gpointer data, gpointer user_data)
 		element->parent = element_root->data;
 
 		node = element_root;
-		basepath = "";
+		basepath = "/device";
 	}
 
 	element->path = g_strdup_printf("%s/%s", basepath, element->name);
 
 	__connman_element_unlock(element);
 
+	if (node == NULL) {
+		connman_error("Element registration for %s failed",
+							element->path);
+		return;
+	}
+
 	DBG("element %p path %s", element, element->path);
 
 	g_node_append_data(node, element);
@@ -1187,7 +1213,7 @@ int connman_element_register(struct connman_element *element,
 	}
 
 	if (element->type == CONNMAN_ELEMENT_TYPE_DHCP)
-		element->ipv4.method = CONNMAN_IPV4_METHOD_DHCP;
+		element->ipv4.method = CONNMAN_IPCONFIG_METHOD_DHCP;
 
 	element->parent = parent;
 
@@ -1302,6 +1328,19 @@ int connman_element_set_enabled(struct connman_element *element,
 	return 0;
 }
 
+static enum connman_service_error convert_error(enum connman_element_error error)
+{
+	switch (error) {
+	case CONNMAN_ELEMENT_ERROR_UNKNOWN:
+	case CONNMAN_ELEMENT_ERROR_FAILED:
+		break;
+	case CONNMAN_ELEMENT_ERROR_DHCP_FAILED:
+		return CONNMAN_SERVICE_ERROR_DHCP_FAILED;
+	}
+
+	return CONNMAN_SERVICE_ERROR_UNKNOWN;
+}
+
 /**
  * connman_element_set_error:
  * @element: element structure
@@ -1312,6 +1351,8 @@ int connman_element_set_enabled(struct connman_element *element,
 void connman_element_set_error(struct connman_element *element,
 					enum connman_element_error error)
 {
+	struct connman_service *service;
+
 	DBG("element %p error %d", element, error);
 
 	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
@@ -1322,6 +1363,9 @@ void connman_element_set_error(struct connman_element *element,
 
 	if (element->driver && element->driver->change)
 		element->driver->change(element);
+
+	service = __connman_element_get_service(element);
+	__connman_service_indicate_error(service, convert_error(error));
 }
 
 int __connman_element_init(DBusConnection *conn, const char *device,
@@ -1382,14 +1426,14 @@ void __connman_element_start(void)
 
 	__connman_connection_init();
 	__connman_ipv4_init();
-	__connman_detect_init();
+	__connman_rfkill_init();
 }
 
 void __connman_element_stop(void)
 {
 	DBG("");
 
-	__connman_detect_cleanup();
+	__connman_rfkill_cleanup();
 	__connman_ipv4_cleanup();
 	__connman_connection_cleanup();
 }
